#ifdef GL_ES
precision highp float;
#endif

varying vec2 vTex;
uniform sampler2D samplerFront;

uniform float uDistance;
uniform float uAngle;
uniform float uOpacity;
uniform float uHue;
uniform float uSaturation;
uniform float uBrightness;
uniform float uBlurRadius;

vec3 rgb2hsv(vec3 c);
vec3 hsv2rgb(vec3 c);

void main(void) {
    // Convert the angle to radians
    float angleRad = uAngle * 3.14159265 / 180.0;

    // Calculate the shadow offset based on the viewport size
    vec2 offset = uDistance * 0.001 * vec2(cos(angleRad), sin(angleRad));

    vec4 shadow = vec4(0.0);
    for(int i = -4; i <= 4; i++) {
        for(int j = -4; j <= 4; j++) {
            vec2 sampledCoord = vTex + offset + vec2(i, j) * (uBlurRadius * 0.01) * 0.001; 
            shadow += texture2D(samplerFront, sampledCoord);
        }
    }
    shadow /= 81.0;
    shadow.a *= uOpacity;

    // Set base shadow color to red
	shadow.rgb = vec3(1.0, 0.0, 0.0);

    // Convert shadow color to HSV
    vec3 shadowHSV = rgb2hsv(shadow.rgb);
    
    // Adjust hue, saturation, and brightness
    shadowHSV.x = mod(shadowHSV.x + uHue / 360.0, 1.0); // Normalize hue to [0, 1] range and ensure it wraps around
	shadowHSV.y = uSaturation;
	shadowHSV.z = uBrightness;

    // Convert back to RGB
    shadow.rgb = hsv2rgb(shadowHSV);
    
    // Get the original color
    vec4 col = texture2D(samplerFront, vTex);

    // Combine shadow with the sprite
    vec4 shadowColor = mix(col, shadow, shadow.a); 

    // Final color
    gl_FragColor = mix(shadowColor, col, col.a);
}

vec3 rgb2hsv(vec3 c) {
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
    
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 c) {
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}